/*
 * Board module
 *
 * Copyright © 1998-2000, 2005, 2007 Guillem Jover <guillem@hadrons.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 *
 */

#include <stdlib.h>

#include "player.h"
#include "board.h"

Board *board;

bool board_init(void)
{
	int i, j;

	board = malloc(sizeof(Board));

	board->turn = 0;
	board->move = board_move;
	board->winner = PLAYER_NONE;

	for (i = 0; i < DIM; i++) {
		board->height[i] = 0;

		for (j = 0; j < DIM; j++) {
			board->grid[j][i].who = PLAYER_BLANK;
			board->grid[j][i].x = i;
			board->grid[j][i].y = j;
			board->grid[j][i].ui_data = NULL;
		}
	}

	return true;
}

bool board_done(void)
{
	free(board);

	return true;
}

bool in_range(index_t a)
{
	return (a >= 0) && (a < DIM);
}

int board_get_y(index_t x)
{
	return board->height[x];
}

bool can_move(index_t x)
{
	index_t y = board_get_y(x);

	return ((y < DIM) &&
		(x < DIM) && (board->grid[y][x].who == PLAYER_BLANK) &&
		((y == 0) ||
		 ((y > 0) && (board->grid[y - 1][x].who != PLAYER_BLANK))));
}

bool board_move(index_t col)
{
	if (can_move(col)) {
		int row = board->height[col]++;
		board->grid[row][col].who = player->symbol;
		if (board->draw_move)
			board->draw_move(col, row);
		board->turn++;
		return true;
	} else
		return false;
}

