/*
 * Board module
 *
 * Copyright © 1998-2000, 2005, 2007 Guillem Jover <guillem@hadrons.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 *
 */

#ifndef BOARD_H
#define BOARD_H

#include "types.h"
#include "player.h"

/* Board dimensions */
#define DIM 7
/* Half board dimensions (in index count) base 0 */
#define HALFDIM 3

typedef struct {
	symbol_t who;
	index_t x, y;
	void *ui_data;
} Cell;

typedef struct {
	Cell grid[DIM][DIM];
	index_t height[DIM];
	count_t turn;
	symbol_t winner;

	bool (*move)(index_t col);
	void (*draw_move)(int x, int y);
} Board;

extern Player *player;
extern Board *board;

extern bool in_range(index_t a);
extern bool can_move(index_t x);
extern bool board_move(index_t x);
extern int board_get_y(index_t x);

extern bool board_init();
extern bool board_done();

#endif

