/*
 * Rules Module
 *
 * Copyright © 1998-2000, 2005 Guillem Jover <guillem@hadrons.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 *
 */

#ifndef RULES_H
#define RULES_H

#include "types.h"
#include "player.h"
#include "board.h"

/* Winner segment dimensions */
#define WIN_DIM 4

typedef enum {
	DIR_H = 0x01,
	DIR_V = 0x02,
	DIR_D = 0x0C,
	DIR_DP = 0x04,		/* Primary	'\'	*/
	DIR_DS = 0x08,		/* Secondary	'/'	*/
} dir_t;

typedef struct {
	count_t priority;	/* turns since other wins */
	dir_t valid_dirs;
	symbol_t *text;		/* pattern */
} Pattern;

extern Pattern win_pattern;

typedef struct Line_st {
	count_t priority;
	index_t x, y;
	dir_t dir;
	count_t size;
	symbol_t line[DIM];
	symbol_t owner;

	struct Line_st *next;
} Line;

#define ENTRIES_IN(a) (sizeof(a) / sizeof((a)[0]))

extern Line *alloc_line(void);
extern void free_line(Line *ln);
extern void copy_line(Line *dst, Line *src);
extern void add_line(Line **list, Line *add);
extern void insert_line(Line **list, Line *in);

extern bool extract_line(index_t line_idx, dir_t dir, Line *ln);
extern symbol_t equ_pattern(symbol_t sym, Pattern *pat, Line* ln, index_t delta);

extern Line *search_hpat(symbol_t sym, Pattern *pat);
extern Line *search_vpat(symbol_t sym, Pattern *pat);
extern Line *search_dpat(symbol_t sym, Pattern *pat);
extern Line *search_pattern(symbol_t sym, Pattern *pat);

/* Begin Inerface to define ... */
extern Move *search_moves(Line *ln);
extern Move *search_traps(Line *ln);

extern bool think(void);		//hard
extern bool computer_play(void);
/* End Inerface to define ... */

#endif

