/*
 * User Interface
 *
 * Copyright © 2005, 2007 Guillem Jover <guillem@hadrons.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 *
 */

#define _GNU_SOURCE

#include <dlfcn.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include "player.h"
#include "board.h"
#include "game.h"
#include "ui.h"

static struct ui_module *ui;
static void *ui_so;

bool ui_load(char *module_name)
{
	char *file_name;
	char *libdir = get_current_dir_name();
	char *error;

	asprintf(&file_name, "%s/ui_%s.so", libdir, module_name);

	ui_so = dlopen(file_name, RTLD_LAZY);
	if (!ui_so) {
		fprintf(stderr, "ui_load: %s\n", dlerror());
		exit(1);
	}

	dlerror();
	ui = dlsym(ui_so, "ui_module");
	error = dlerror();
	if (error) {
		fprintf(stderr, "ui_load: %s\n", error);
		exit(1);
	}

	free(libdir);
	free(file_name);

	return true;
}

bool ui_init(int argc, char **argv)
{
	printf("Loading ui module: %s.\n", ui->name);

	if (!ui->loop) {
		board->draw_move = ui->draw_board_move;

		player->get_move = ui->get_user_move;
		player->other->get_move = ui->get_user_move;
	}

	if (ui->init)
		ui->init(argc, argv);

	return true;
}

bool ui_loop(void)
{
	if (ui->loop)
		return ui->loop();
	else {
		ui->display_board();
		do {
			int x = player->get_move();

			if (!can_move(x))
				continue;

			board->move(x);
			swap_player();
			ui->display_board();
		} while (!game_over());

		ui->display_winner();

		return true;
	}
}

bool ui_done(void)
{
	if (ui->done)
		ui->done();

	dlclose(ui_so);

	return true;
}

