/*
 * Xt output routines
 *
 * User Interface Module
 * Copyright © 1998-2000, 2005 Guillem Jover <guillem@hadrons.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 *
 */

#include <X11/Intrinsic.h>
#include <X11/Shell.h>
#include <X11/StringDefs.h>

#include <X11/Xaw/Form.h>
#include <X11/Xaw/Command.h>

#include <X11/xpm.h>

#include <stdlib.h>
#include <stdio.h>

#include "player.h"
#include "board.h"
#include "game.h"
#include "ui.h"

typedef struct {
	Pixmap image;
} Xt_Player;

typedef struct {
	Widget button;
} Xt_Cell;

static Display *display;
static Drawable drawable;
static XtAppContext app_context;
static Widget app;
static Widget top;
static Atom wm_delete_window;

#include "point.xpm"
#include "cross.xpm"

static void Callback(Widget w, XtPointer user_data, XtPointer call_data)
{
	Cell *cell = (Cell *)user_data;

	/* XXX: Make current player visible somehow:
	 *      - changing the cursor
	 *      - a side box
	 */
	if (can_move(cell->x)) {
		int y = board_get_y(cell->x);
		Xt_Cell *xt_cell = board->grid[y][cell->x].ui_data;
		Xt_Player *xt_player = player->ui_data;

		XtVaSetValues(xt_cell->button,
			XtNbackgroundPixmap, xt_player->image, NULL);
		XtOverrideTranslations(xt_cell->button,
			XtParseTranslationTable("<Btn1Down>: \n<Btn1Up>:"));

		board->move(cell->x);

		swap_player();

		if (game_over()) {
			/* XXX: Display a window with the winner */
			/* XXX: Exit properly, cleaning all stuff */
			exit(0);
		}
	}
}

static void Quit(Widget w, XEvent *event, String *params, Cardinal *nparams)
{
	if (event->type == ClientMessage &&
	    event->xclient.data.l[0] == wm_delete_window)
	{
		XtCloseDisplay(XtDisplay(w));
		exit(0);
	}
}

static XtActionsRec actions_4inline[] = {
	{"quit", Quit},
};

static bool xt_init(int app_argc, char **app_argv)
{
	Xt_Player *xt_player;
	Xt_Cell *xt_cell;
	Widget wh, wv;
	int i, j;

	app = XtVaOpenApplication(&app_context, "4inline", NULL , 0,
		&app_argc, app_argv, NULL, applicationShellWidgetClass,
		NULL);

	top = XtVaCreateManagedWidget("top", formWidgetClass, app,
		XtNbackground, 0x000000,
		XtNheight, DIM * 33,
		XtNwidth, DIM * 33,
		XtNdefaultDistance, 1,
		NULL);
	XtRealizeWidget(app);

	display = XtDisplay(app);
	drawable = XtWindow(app);

	xt_player = malloc(sizeof(Xt_Player));
	XpmCreatePixmapFromData(display, drawable, point_xpm, &(xt_player->image),
		NULL, NULL);
	player->ui_data = (Player *)xt_player;

	xt_player = malloc(sizeof(Xt_Player));
	XpmCreatePixmapFromData(display, drawable, cross_xpm, &(xt_player->image),
		NULL, NULL);
	player->other->ui_data = (Player *)xt_player;

	for (j = DIM - 1; j >= 0; j--) for (i = 0; i < DIM; i++)
	{
		if (j == DIM - 1)
			wv = NULL;
		else
			wv = ((Xt_Cell *)board->grid[j + 1][i].ui_data)->button;
		if (i == 0)
			wh = NULL;
		else
			wh = ((Xt_Cell *)board->grid[j][i - 1].ui_data)->button;

		xt_cell = malloc(sizeof(Xt_Cell));
		xt_cell->button = XtVaCreateManagedWidget("button", commandWidgetClass, top,
			XtNbackground, 0xffffff,
			XtNlabel, "",
			XtNborderWidth, 0,
			XtNresizable, TRUE,
			XtNheight, 32, XtNwidth, 32,
			XtNfromHoriz, wh,
			XtNfromVert, wv,
			XtNbottom, XawChainTop,
			XtNtop, XawChainTop,
			XtNleft, XawChainLeft,
			XtNright, XawChainLeft,
			NULL);
		XtAddCallback(xt_cell->button, "callback", Callback,
			(XtPointer)&board->grid[j][i]);
		board->grid[j][i].ui_data = xt_cell;
	}

	XtRealizeWidget(app);

	wm_delete_window = XInternAtom(display, "WM_DELETE_WINDOW", True);
	XSetWMProtocols(display, XtWindow(app), &wm_delete_window, 1);
	XtAppAddActions(app_context, actions_4inline, XtNumber(actions_4inline));
	XtOverrideTranslations(app,
		XtParseTranslationTable("<Message>WM_PROTOCOLS: quit()"));

        return true;
}

static bool xt_loop(void)
{
	XtAppMainLoop(app_context);

	return true;
}

static bool xt_done(void)
{
	int i, j;

	free(player->ui_data);
	free(player->other->ui_data);

	for (i = 0; i < DIM; i++)
		for (j = 0; j < DIM; j++)
			free(board->grid[j][i].ui_data);

	return true;
}

struct ui_module ui_module = {
	.name = "Xt",
	.init = xt_init,
	.done = xt_done,
	.loop = xt_loop,
};

