/*
 * GTK output routines
 *
 * User Interface Module
 * Copyright © 2007 Guillem Jover <guillem@hadrons.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 *
 */

#include <gdk-pixbuf/gdk-pixbuf-core.h>
#include <gtk/gtk.h>

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>

#include "player.h"
#include "board.h"
#include "game.h"
#include "ui.h"

typedef struct {
	GtkWidget *image;
} UI_Player;

typedef struct {
	GtkWidget *button;
} UI_Cell;

static GtkWidget *app;
static GtkWidget *table;

#include "point.xpm"
#include "cross.xpm"

static void
show_winner(UI_Player *winner)
{
	GtkWidget *note;
	GtkTextBuffer *buf;

	/* FIXME: this does not work yet */
	note = gtk_text_view_new();
	buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(note));
	gtk_text_buffer_set_text(buf, "Player won!", -1);

	gtk_container_remove(GTK_CONTAINER(app), table);
	gtk_container_add(GTK_CONTAINER(app), note);

	gtk_widget_show_all(app);

	sleep(5);
}

static void
clicked(GtkWidget *widget, gpointer data)
{
	Cell *cell = (Cell *)data;

	/* XXX: Make current player visible somehow:
	 *      - changing the cursor
	 *      - a side box
	 */
	if (can_move(cell->x)) {
		int y = board_get_y(cell->x);
		UI_Cell *mod_cell = board->grid[y][cell->x].ui_data;
		UI_Player *mod_player = player->ui_data;

		gtk_button_set_image(GTK_BUTTON(mod_cell->button),
		                     mod_player->image);
		// g_signal disconnect signal

		board->move(cell->x);

//		gtk_widget_show(mod_cell->button);

		swap_player();

		if (game_over()) {
			show_winner(mod_player);
			/* XXX: Exit properly, cleaning all stuff */
			exit(0);
		}
	}
}

static UI_Player *
create_ui_player(char **data)
{
	GdkPixbuf *pixbuf;
	UI_Player *ui_player;

	pixbuf = gdk_pixbuf_new_from_xpm_data((const char **)data);

	ui_player = malloc(sizeof(UI_Player));
	ui_player->image = gtk_image_new_from_pixbuf(pixbuf);

	return ui_player;
}

static bool
mod_gtk_init(int app_argc, char **app_argv)
{
	int i, j;

	gtk_init(&app_argc, &app_argv);

	app = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(app), "4inline");
//	gtk_window_set_default_size(GTK_WINDOW(app), DIM * 33, DIM * 33);
//	gtk_window_set_resizable(GTK_WINDOW(app), false);

	player->ui_data = create_ui_player(point_xpm);
	player->other->ui_data = create_ui_player(cross_xpm);

	table = gtk_table_new(DIM, DIM, true);

	gtk_table_set_col_spacings(GTK_TABLE(table), 1);
	gtk_table_set_row_spacings(GTK_TABLE(table), 1);

	gtk_container_add(GTK_CONTAINER(app), table);

	for (j = 0; j < DIM; j++) for (i = 0; i < DIM; i++) {
		UI_Cell *mod_cell;
		GtkWidget *button;

		mod_cell = malloc(sizeof(UI_Cell));

		button = gtk_button_new();
//		gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
		gtk_widget_set_size_request(button, 40, 40);
//		gtk_container_set_border_width(button, 0);

		gtk_table_attach_defaults(GTK_TABLE(table), button,
		                          i, i + 1, DIM - j - 1, DIM - j);

		g_signal_connect(button, "clicked", G_CALLBACK(clicked),
		                 (gpointer)&board->grid[j][i]);

		mod_cell->button = button;
		board->grid[j][i].ui_data = mod_cell;
	}

	g_signal_connect(app, "destroy", G_CALLBACK(gtk_main_quit), NULL);

	return true;
}

static bool
mod_gtk_loop(void)
{
	gtk_widget_show_all(app);

	gtk_main();

	return true;
}

static bool
mod_gtk_done(void)
{
	int i, j;

	// FIXME: free the images
	free(player->ui_data);
	free(player->other->ui_data);

	gtk_widget_destroy(app);

	for (i = 0; i < DIM; i++)
		for (j = 0; j < DIM; j++)
			// FIXME: free the button
			free(board->grid[j][i].ui_data);

	return true;
}

struct ui_module ui_module = {
	.name = "gtk",
	.init = mod_gtk_init,
	.done = mod_gtk_done,
	.loop = mod_gtk_loop,
};

