/*
 * terminal output routines
 *
 * User Interface Module
 * Copyright © 1998-2000, 2005 Guillem Jover <guillem@hadrons.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 *
 */

#include <stdio.h>

#include "player.h"
#include "board.h"
#include "game.h"
#include "ui.h"

#define MAXBUF 256

static void term_display_board(void)
{
	int i, j;

	puts(" +---+---+---+---+---+---+---+");
	for (j = DIM - 1; j >= 0; j--) {
		printf("%u|", j + 1);
		for (i = 0; i < DIM; i++) {
			printf(" %c |", board->grid[j][i].who);
		}
		puts("\n +---+---+---+---+---+---+---+");
	}
	for (i = 1; i <= DIM; i++)
		printf("   %u", i);
	puts("");
}

static void term_display_winner(void)
{
	printf("Player [%c] won!\n", board->winner);
}

static int term_get_user_move(void)
{
	char buf[MAXBUF];
	int ret;
	index_t col;

	do {
		printf("Column position (player [%c]): ", player->symbol);
		ret = scanf("%lu", &col);
		fgets(buf, MAXBUF, stdin);
	} while (!ret || (ret && ((col < 1) || (col > DIM))));
	col--;

	return col;
}

struct ui_module ui_module = {
	.name = "term",
	.display_board = term_display_board,
	.display_winner = term_display_winner,
	.get_user_move = term_get_user_move,
};

