/*
 * vga output routines
 *
 * User Interface Module
 * Copyright © 1998-2000, 2005 Guillem Jover <guillem@hadrons.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 *
 */

#include <dos.h>
#include "types.h"

/* For Graphics representation */

u8 blank[10, 10] = {
	{0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
	{0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
	{0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
	{0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
	{0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
	{0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
	{0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
	{0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
	{0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
	{0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
};

u8 point[10, 10] = {
	{0, 0, 0, 0, 1, 1, 0, 0, 0, 0},
	{0, 0, 1, 1, 2, 2, 1, 1, 0, 0},
	{0, 1, 2, 2, 3, 3, 2, 2, 1, 0},
	{0, 1, 2, 3, 4, 4, 3, 2, 1, 0},
	{1, 2, 3, 4, 5, 5, 4, 3, 2, 1}
	{1, 2, 3, 4, 5, 5, 4, 3, 2, 1},
	{0, 1, 2, 3, 4, 4, 3, 2, 1, 0},
	{0, 1, 2, 2, 3, 3, 2, 2, 1, 0},
	{0, 0, 1, 1, 2, 2, 1, 1, 0, 0},
	{0, 0, 0, 0, 1, 1, 0, 0, 0, 0}
};

u8 cross[10, 10] = {
	{6, 6, 0, 0, 0,  0,  0, 0, 6, 6}
	{6, 7, 7, 0, 0,  0,  0, 7, 7, 6}
	{0, 7, 8, 8, 0,  0,  8, 8, 7, 0}
	{0, 0, 8, 9, 9,  9,  9, 8, 0, 0}
	{0, 0, 0, 9, 10, 10, 9, 0, 0, 0}
	{0, 0, 0, 9, 10, 10, 9, 0, 0, 0}
	{0, 0, 8, 9, 9,  9,  9, 8, 0, 0}
	{0, 7, 8, 8, 0,  0,  8, 8, 7, 0}
	{6, 7, 7, 0, 0,  0,  0, 7, 7, 6}
	{6, 6, 0, 0, 0,  0,  0, 0, 6, 6}
};


#define MAX_COLORS	256

typedef struct {
	u8 r;
	u8 g;
	u8 b;
} color_RGB;

typedef color_RGB palette_RGB[MAX_COLORS];


typedef struct {
	count_t width, length;
	void *data;
} Image;

/*
 * Hardware Level Functions
 */
u32 vga_init();
u32 vga_done();
u32 wait_vr();
u32 get_pixel(u32 x, u32 y, index_t *color);
u32 put_pixel(u32 x, u32 y, index_t *color);
u32 get_area(u32 x1, u32 y1, u32 x2, u32 y2, Image *img);
u32 put_area(u32 x1, u32 y1, u32 x2, u32 y2, Image *img);
u32 get_screen(Image *img);
u32 put_screen(Image *img);
u32 draw_line(u32 x1, u32 y1, u32 x2, u32 y2);
u32 draw_hline(u32 x, u32 y, u32 length);
u32 draw_vline(u32 x, u32 y, u32 length);
u32 get_pal_item(color_RGB *color);
u32 put_pal_item(color_RGB *color);
u32 get_pal(palette_RGB *pal);
u32 put_pal(palette_RGB *pal);

/*
 * Visual FX Functions
 */

void rotate_pal(palette_RGB *pal);
void black_pal();
void fadeout();
void fadein(palette_RGB *pal);

/*
 * Figure Drawing Functions
 */

void draw_circle();
void draw_box();
void draw_rect();

/*
 * Image Drawing Functions
 */

void save_screen(ImageFile *img);
void load_screen(ImageFile *img);
void load_pcx(ImageFile *pcx);


/*
 * Implementation ...
 */

u32 vga_init(void)
{
	union REGS r;

	r.ax = 0x13;
	int86(0x10, &r, &r);

	return 0;
}

u32 vga_done(void)
{
	union REGS r;

	r.ax = 0x3;
	int86(0x10, &r, &r);

	return 0;
}

/*
Procedure EsperaVR; Assembler;
   asm
      mov dx,$3da
      @1:
      in al,dx
      test al,8
      jnz @1
      @2:
      in al,dx
      test al,8
      jz @2
   end;

Procedure PosaPixel(x,y:integer;color:byte); Assembler;
   asm
      mov ax,$a000
      mov es,ax
      mov al,[Color]
      mov bx,[X]
      mov dx,[Y]
      xchg dh,dl
      mov di,dx
      shr di,2
      add di,dx
      add di,bx
      mov es:[di],al
   end;

Procedure ObtePixel(x,y:integer;var color:byte);Assembler;
   asm
      mov ax,$a000
      mov es,ax
      mov bx,[X]
      mov dx,[Y]
      xchg dh,dl
      mov di,dx
      shr di,2
      add di,dx
      add di,bx
      mov al,es:[di]
      les di,[Color]
      stosb
   end;

Procedure OmplePantalla(color:byte);Assembler;
   asm
      mov ax,$a000
      mov es,ax
      mov di,0
      mov al,[Color]
      mov ah,al
      mov cx,32000
      cld
      rep stosw
   end;

Procedure PosaColPal(Color,R,G,B:byte);Assembler;
   asm
      mov al,[Color]
      mov dx,3c8h
      out dx,al
      inc dx
      mov al,[R]
      out dx,al
      mov al,[G]
      out dx,al
      mov al,[B]
      out dx,al
   end;

Procedure ObteColPal(Color:byte;var R,G,B:byte);Assembler;
   asm
      mov al,[Color]
      mov dx,$3c7
      out dx,al
      inc dx
      inc dx
      in al,dx
      les di,[R]
      stosb
      in al,dx
      les di,[G]
      stosb
      in al,dx
      les di,[B]
      stosb
   end;

Procedure PosaPal(paleta:paletaRGB);
   var color,r,g,b:byte;
   begin
       for color:=0 to MAX_COLORS-1 do
          PosaColPal(color,paleta[color].r,paleta[color].g,paleta[color].b);
   end;

Procedure ObtePal(var paleta:paletaRGB);
   var color,r,g,b:byte;
   begin
       for color:=0 to MAX_COLORS-1 do
          ObteColPal(color,paleta[color].r,paleta[color].g,paleta[color].b);
   end;

Procedure GuardaPal(Var arxiu: File; paleta: paletaRGB);{?}
   begin
      BlockWrite(arxiu, paleta, MAX_COLORS);
   end;

Procedure CarregaPal(Var arxiu: File; Var paleta: paletaRGB);{?}
   begin
      BlockRead(arxiu, paleta, MAX_COLORS);
   end;


Procedure PaletaNegra;
   var i:byte;
   begin
      for i:=0 to 255 do PosaColPal(i,0,0,0);
   end;

Procedure FadeOut;
  var x,y:byte;
      r,g,b:byte;
  begin
     for x:=1 to 64 do begin
        EsperaVR;
        for y:=0 to 255 do begin
            ObteColPal(y,r,g,b);
            if r>0 then dec(r);
            if g>0 then dec(g);
            if b>0 then dec(b);
            PosaColPal(y,r,g,b);
        end;
     end;
  end;

Procedure FadeIn(PaletaFinal:PaletaRGB);
  var x,y:byte;
      r,g,b:byte;
  begin
     for x:=1 to 64 do begin
        EsperaVR;
        for y:=0 to 255 do begin
            ObteColPal(y,r,g,b);
            if r<paletafinal[y].r then inc(r);
            if g<paletafinal[y].g then inc(g);
            if b<paletafinal[y].b then inc(b);
            PosaColPal(y,r,g,b);
        end;
     end;
  end;

Procedure RotaPal(var paleta:paletaRGB);
   var x:integer;
       tmp:colorRGB;
   begin
       EsperaVR;
       move(paleta[255],tmp,3);
       move(paleta[0],paleta[1],255*3);
       move(tmp,paleta[0],3);
       for x:=0 to 255 do PosaColPal(x,paleta[x].r,paleta[x].g,paleta[x].b);
   end;

Procedure Linia(x1,y1,x2,y2,Color:integer);
   Function Sgn(a:real):integer;
      begin
         if a>0 then sgn:=+1;
         if a<0 then sgn:=-1;
         if a=0 then sgn:=0;
      end;

   var u,s,v,d1x,d1y,d2x,d2y,m,n:real;
       i:integer;
   begin
      u:=y2-x1;
      v:=x2-y1;
      d1x:=sgn(u);
      d1y:=sgn(v);
      d2x:=sgn(u);
      d2y:=0;
      m:=abs(u);
      n:=abs(v);
      if not (m>n) then begin
         d2x:=0 ;
         d2y:=sgn(v);
         m:=abs(v);
         n:=abs(u);
      end;
      s:=int(m/2);
      for i:=0 to round(m) do begin
         PosaPixel(x1,y1,color);
         s:=s+n;
         if not (s<m) then begin
            s:=s-m;
            x1:=x1+round(d1x);
            y1:=y1+round(d1y);
         end else begin
            x1:=x1+round(d2x);
            y1:=y1+round(d2y);
         end;
      end;
   end;

Procedure LiniaH( X, Y, Tamany: Integer; Color: Byte); Assembler;
   asm
      mov ax,$a000
      mov es,ax
      mov bx,[X]
      mov dx,[Y]
      xchg dh,dl
      mov di,dx
      shr di,2
      add di,dx
      add di,bx
      mov al,[Color]
      mov ah,al
      mov cx,[Tamany]
      rep stosb
   end;

Procedure LiniaH32bit( X, Y, Tamany: Integer; Color: Byte); Assembler;
   asm
	mov ax, $a000
	mov es, ax
	mov bx, [Y]
	mov di, [Y_list+bx]
	add di, [X]
	mov bl, [Color]
	mov bh, bl
	mov ax, bx
	bswap eax
	mov ax, bx
	mov ebx, [Tamany]
	mov ecx, ebx
	shr ecx, 2
	rep stosl
	mov ecx, ebx
	and ecx, 3
	rep stosb
   end;

Procedure LiniaV( X, Y, Tamany: Integer; Color: Byte); Assembler;
   asm
	mov ax,$a000
	mov es,ax
	mov bx,[X]
	mov dx,[Y]
	xchg dh,dl
	mov di,dx
	shr di,2
	add di,dx
	add di,bx
	mov al,[Color]
	mov cx,[Tamany]
   @Bucle:
	stosb
	add di,200
	loop @Bucle
   end;

Procedure Cercle(Cx,Cy,Radi:integer;Color:Byte);
   var deg:real;
       x,y:integer;
   begin
      deg:=0;
      repeat
         X:=round(radi*cos(deg));
         Y:=round(radi*sin(deg));
         posapixel(X+Cx,Y+Cy,color);
         deg:=deg+0.005;
      until (deg>6.4);
   end;

Procedure CreaRectangle(x1,y1,x2,y2:integer;color:byte);
   var y, len:integer;
   begin
	len := x2 - x1;
	for y:=y1 to y2 do LiniaH(x1,y,len,color);
   end;

Procedure CreaMarc(x1,y1,x2,y2:integer;color:byte);
   var i:integer;
   begin
	LiniaH(x1, y1, x2 - x1, color);
	LiniaH(x1, y2, x2 - x1, color);
	LiniaV(x1, y1+1, y2 - y1 - 2, color);
	LiniaV(x2, y1+1, y2 - y1 - 2, color);
   end;

Procedure GuardaPantalla(var arxiu:file);
   begin
      Blockwrite(arxiu,mem[$a000:0],64000);
   end;

Procedure CarregaPantalla(var arxiu:file);
   begin
      Blockread(arxiu,mem[$a000:0],64000);
   end;

Procedure CarregaPCX(var arxiu:file;resultat:byte);
   var Long:longint;
       LongX,LongY:word;
       zx,zy:word;
       x1,y1:word;
       x2,y2:word;
       rep:word;
       index:word;
       dat:byte;
       ch:byte;
       R,G,B:byte;
   begin
      Long:=filesize(arxiu);           {Carrega la capçalera}
      Seek(arxiu,0);
      Blockread(arxiu,dat,1);
      if (dat<10) then begin
         Resultat:=pcxErrorArxiu;
         Exit;
      end;
      Blockread(arxiu,dat,1);
      if (dat<5) then begin
         Resultat:=pcxErrorVersio;
         Exit;
      end;
      Seek(arxiu,3);
      Blockread(arxiu,dat,1);
      if (dat<8) then begin
         Resultat:=pcxErrorBPP;
         Exit;
      end;
      Blockread(arxiu,x1,2);
      Blockread(arxiu,y1,2);
      Blockread(arxiu,x2,2);
      Blockread(arxiu,y2,2);
      LongX:=x2-x1;
      LongY:=y2-y1;
      Seek(arxiu,128);                 {Carrega el grafic}
      zx:=0; zy:=0; rep:=1;
      while zy<longy do begin
         Blockread(arxiu,dat,1);
         if dat<=192 then begin
            PosaPixel(zx,zy,dat);
            inc(zx);
            if zx>longx then begin
               zx:=0;
               inc(zy);
            end;
         end else begin
            rep:=(dat and 63);
            Blockread(arxiu,ch,1);
            for index:=1 to rep do begin
               PosaPixel(zx,zy,ch);
               inc(zx);
               if zx>longx then begin
                  zx:=0;
                  inc(zy);
               end;
            end;
         end;
      end;
      seek(arxiu,long-768);                {Carrega la paleta}
      for index:=0 to 255 do begin
         Blockread(arxiu,r,1);
         Blockread(arxiu,g,1);
         Blockread(arxiu,b,1);
         r:=63*r div 255;
         g:=63*g div 255;
         b:=63*b div 255;
         PosaColPal(index,R,G,B);
      end;
      Resultat:=pcxBe;
   end;

End.
*/

